package test.org.lantsovanton;


import org.lantsovanton.abstraction.model.ICoordinate;
import org.lantsovanton.abstraction.model.IGame;
import org.lantsovanton.abstraction.model.IMove;
import org.lantsovanton.abstraction.model.IPosition;
import org.lantsovanton.abstraction.model.impl.Chip;
import org.lantsovanton.abstraction.model.impl.Coordinate;
import org.lantsovanton.abstraction.model.impl.Move;
import org.lantsovanton.abstraction.model.impl.Position;
import org.lantsovanton.games.noughtsandcrosses.NoughtsAndCrosses;

import junit.framework.TestCase;

public class NoughtAndCrossesTest extends TestCase {
	
	public void testInitialPosition(){
		//initial
		IGame game = new NoughtsAndCrosses();
		IPosition position = game.initialPosition();
		IPosition idealPosition = new Position( 3, 3, 1 );
		assertEquals( idealPosition, position );
	}
	
	public void testIsMovePossible(){
		// in start of game
		//move unpossible if number of player in position  != number of player in move
		IGame game = new NoughtsAndCrosses();
		IPosition position = game.initialPosition();
		for( int i = 0; i < 3; i++  ){
			for( int j = 0; j < 3; j++ ){
				IMove move1 = new Move( i, j, 1);
				IMove move2 = new Move( i, j, 2);
				assertEquals( false, game.isMovePosible( position, move2 ) );
				assertEquals( true, game.isMovePosible( position, move1 ) );
			}
		}
		
		//move unpossible if it try set chip on exist chip
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		position.set( 1, 1, new Chip( 1 ) );
		position.setPlayer(2);
		IMove move = new Move( 1, 1, 2);
		assertEquals( false, game.isMovePosible( position, move ) );	
		
		///if game finish move unpossible
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 1, 1, new Chip( 1 ) );
		position.set( 2, 2, new Chip( 1 ) );
		position.setPlayer(2);
		move = new Move( 1, 0, 2);
		assertEquals( false, game.isMovePosible( position, move ) );	
	}
	
	public void testwinner(){
		IGame game = new NoughtsAndCrosses();
		IPosition position = game.initialPosition();
		
		//goriontal 
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 0, 1, new Chip( 1 ) );
		position.set( 0, 2, new Chip( 1 ) );
		
		assertEquals( true, game.isWinner( position, 1 ) );
		assertEquals( false, game.isWinner( position, 2 ) );
		//
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		position.set( 1, 0, new Chip( 1 ) );
		position.set( 1, 1, new Chip( 1 ) );
		position.set( 1, 2, new Chip( 1 ) );
		
		assertEquals( true, game.isWinner( position, 1 ) );
		assertEquals( false, game.isWinner( position, 2 ) );
		//
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		position.set( 2, 0, new Chip( 1 ) );
		position.set( 2, 1, new Chip( 1 ) );
		position.set( 2, 2, new Chip( 1 ) );
		
		assertEquals( true, game.isWinner( position, 1 ) );
		assertEquals( false, game.isWinner( position, 2 ) );
		
		//verticall
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 1, 0, new Chip( 1 ) );
		position.set( 2, 0, new Chip( 1 ) );
		
		assertEquals( true, game.isWinner( position, 1 ) );
		assertEquals( false, game.isWinner( position, 2 ) );
		//
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		position.set( 0, 1, new Chip( 1 ) );
		position.set( 1, 1, new Chip( 1 ) );
		position.set( 2, 1, new Chip( 1 ) );
		
		assertEquals( true, game.isWinner( position, 1 ) );
		assertEquals( false, game.isWinner( position, 2 ) );
		//
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		position.set( 0, 2, new Chip( 1 ) );
		position.set( 1, 2, new Chip( 1 ) );
		position.set( 2, 2, new Chip( 1 ) );
		
		assertEquals( true, game.isWinner( position, 1 ) );
		assertEquals( false, game.isWinner( position, 2 ) );
		
		//digonal
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 1, 1, new Chip( 1 ) );
		position.set( 2, 2, new Chip( 1 ) );
		
		assertEquals( true, game.isWinner( position, 1 ) );
		assertEquals( false, game.isWinner( position, 2 ) );
		//
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		
		position.set( 0, 2, new Chip( 1 ) );
		position.set( 1, 1, new Chip( 1 ) );
		position.set( 2, 0, new Chip( 1 ) );
		
		assertEquals( true, game.isWinner( position, 1 ) );
		assertEquals( false, game.isWinner( position, 2 ) );
	}
	
	public void testgamefinish(){ 
		
		//any one win
		IGame game = new NoughtsAndCrosses();
		IPosition position = game.initialPosition();
		
		
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 0, 1, new Chip( 1 ) );
		position.set( 0, 2, new Chip( 1 ) );
		
		assertEquals( true, game.isGameFinish( position ) );
		//
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		
		position.set( 0, 2, new Chip( 2 ) );
		position.set( 1, 1, new Chip( 2 ) );
		position.set( 2, 0, new Chip( 2 ) );
		
		assertEquals( true, game.isGameFinish( position ) );
		
		//board contain 9 chips, nobody win
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 0, 1, new Chip( 2 ) );
		position.set( 0, 2, new Chip( 2 ) );
		position.set( 1, 0, new Chip( 2 ) );
		position.set( 2, 0, new Chip( 2 ) );
		position.set( 1, 1, new Chip( 1 ) );
		position.set( 1, 2, new Chip( 1 ) );
		position.set( 2, 1, new Chip( 1 ) );
		position.set( 2, 2, new Chip( 2 ) );
		
		assertEquals( true, game.isGameFinish( position ) );
		
		//game not finish
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		position.set( 0, 0, new Chip( 1 ) );
		position.set( 0, 1, new Chip( 2 ) );
		assertEquals( false, game.isGameFinish( position ) );
		
		//board empty
		game = new NoughtsAndCrosses();
		position = game.initialPosition();
		assertEquals( false, game.isGameFinish( position ) );
	}
	
	public void testdomove(){
		IGame game = new NoughtsAndCrosses();
		IPosition position = game.initialPosition();
		for( int i = 0; i < 3; i++  ){
			for( int j = 0; j < 3; j++ ){
				IPosition positionideal = new Position( 3, 3, 2 );
				position = game.initialPosition();
				IMove move = new Move( i, j, 1);	
				game.doMove(position,move);
				positionideal.set( i, j, new Chip( 1 ) );
				assertEquals( positionideal.toString() , position.toString() );
			}
		}
	}
	
	public void testgetName(){
		IGame game = new NoughtsAndCrosses();
		assertEquals( "NoughtsAndCrosses", game.getName() );
	}
	
	public void testgetMove(){
		//move possible
		IGame game = new NoughtsAndCrosses();
		IPosition position = game.initialPosition();
		ICoordinate coordinate = new Coordinate(1,2);
		IMove move = new Move( 1,2,1 );
		assertEquals( move, game.getMove( position, coordinate ) );
		
		//move unpossible
		position.set( 1, 2 , new Chip( 1 ) );
		assertEquals( null, game.getMove( position, coordinate ) );
	}
	
	

}
